@@ -3,8 +3,8 @@
 ;; Copyright 2022 James Nguyen
 
 ;; Author: James Nguyen <james@jojojames.com>
-;; Package-Version: 2.1
-;; Package-Revision: dd66f9640079
+;; Package-Version: 20260216.27
+;; Package-Revision: 17ca387c077f
 ;; Package-Requires: ((emacs "28.2") (flx "0.5") (compat "30.0.0.0"))
 ;; Keywords: matching
 ;; Homepage: https://github.com/jojojames/fussy
@@ -146,6 +146,7 @@ candidates. Keep N at 0 or more for performance."
 
 (defcustom fussy-score-threshold-to-filter-alist
   '((flx-score . -100)
+    (fussy-flx-rs-score . -100)
     (fussy-fuz-score . -100)
     (fussy-fuz-bin-score . -100)
     (fussy-fzf-native-score . 0)
@@ -381,6 +382,11 @@ string may go from something like abcX to abcR where X was the multibyte char
 that is not usable with the above scoring backends and R is a random ascii
 character encoded from X.
 
+This is set to nil if `fussy-setup' is called as we use the workaround
+described here:
+https://github.com/axelf4/hotfuzz?tab=readme-ov-file#dynamic-module
+You can set this again if another encoding proves to be a problem.
+
 For more information: \(https://github.com/minad/consult/issues/585\)"
   :type `(choice
           (const :tag "Remove Tofu"
@@ -389,6 +395,8 @@ For more information: \(https://github.com/minad/consult/issues/585\)"
                  ,#'fussy-without-unencodeable-chars)
           (const :tag "Convert to Unibyte"
                  ,#'fussy-encode-coding-string)
+          (const :tag "Don't convert"
+                 nil)
           (function :tag "Custom function"))
   :group 'fussy)
 
@@ -459,6 +467,21 @@ https://lists.gnu.org/archive/html/help-gnu-emacs/2008-06/msg00087.html"
        (message "%.06f" (float-time (time-since time)))
        result)))
 
+;;
+;; (@* "defsubst" )
+;;
+
+
+(defsubst fussy-encode-coding-string (string)
+  "Call `encode-coding-string' for STRING."
+  (encode-coding-string string 'utf-8 t))
+
+(defsubst fussy-without-bad-char (str)
+  "Return STR without bad characters in them."
+  (or (and fussy-remove-bad-char-fn
+           (funcall fussy-remove-bad-char-fn str))
+      str))
+
 ;;
 ;; (@* "Constants and Variables" )
 ;;
@@ -548,7 +571,14 @@ Implement `all-completions' interface with additional fuzzy / `flx' scoring."
           ;; (message "%s from hash with length %d"
           ;;          string (length cached-all))
           ;; (fussy--print-hash-table fussy--all-cache)
-          (nconc cached-all (length prefix)))
+          (nconc (fussy--highlight-collection
+                  (if (fussy--orderless-p)
+                      (fussy--recreate-orderless-pattern
+                       string table pred point)
+                    (fussy--recreate-regex-pattern
+                     beforepoint afterpoint bounds))
+                  cached-all)
+                 (length prefix)))
       (pcase
           (while-no-input
             (pcase-let*
@@ -556,7 +586,7 @@ Implement `all-completions' interface with additional fuzzy / `flx' scoring."
                   (if-let ((cached-all
                             (and
                              fussy-use-cache
-                             (> (length string) 0)
+                             (length> string 0)
                              ;; e.g. ~/.emacs.d/url/ should not use entry from "~/.emacs.d/url".
                              (not (string-suffix-p "/" string))
                              (cl-copy-list
@@ -580,10 +610,10 @@ Implement `all-completions' interface with additional fuzzy / `flx' scoring."
               ;;           'fussy-all-completions
               ;;           string prefix infix (or all '("nada")) pattern))
               (when all
-                (if (or (> (length infix) fussy-max-query-length)
+                (if (or (length> infix fussy-max-query-length)
                         (string= infix ""))
                     (fussy--highlight-collection pattern all)
-                  (if (< (length all) fussy-max-candidate-limit)
+                  (if (length< all fussy-max-candidate-limit)
                       (fussy--highlight-collection
                        pattern
                        (fussy-outer-score all infix cache))
@@ -606,21 +636,8 @@ Implement `all-completions' interface with additional fuzzy / `flx' scoring."
                        ;; Compute all of the fuzzy scores only for candidates.
                        (fussy--highlight-collection
                         pattern
-                        (fussy-outer-score
-                         (reverse candidates-to-score)
-                         infix cache))
-                       ;; Add the unsorted candidates.
-                       ;; We could highlight these too,
-                       ;; (e.g. with `fussy--highlight-collection') but these are
-                       ;; at the bottom of the pile of candidates.
-                       (if fussy-filter-unscored-candidates
-                           (let ((r (car (funcall fussy-default-regex-fn infix))))
-                             (if r
-                                 (cl-remove-if-not
-                                  (lambda (c) (string-match-p r c))
-                                  unscored-candidates)
-                               unscored-candidates))
-                         unscored-candidates))))))))
+                        (fussy-outer-score candidates-to-score infix cache))
+                       unscored-candidates)))))))
         ('nil
          ;; (message "fn: %S nil" 'fussy-all-completions)
          nil)
@@ -631,16 +648,14 @@ Implement `all-completions' interface with additional fuzzy / `flx' scoring."
          ;; (message (format "fn: %S collection: %s"
          ;;                  'fussy-all-completions collection))
          ;; Collection can be 0 when there are no candidates returned.
-         (if (consp collection)
-             (progn
-               (when fussy-use-cache
-                 ;; (message "putting %s into hash with coll length %s"
-                 ;;          string (length collection))
-                 ;; (fussy--print-hash-table fussy--all-cache)
-                 (puthash string (cl-copy-list collection)
-                          fussy--all-cache))
-               (nconc collection (length prefix)))
-           nil))))))
+         (when (consp collection)
+           (when fussy-use-cache
+             ;; (message "putting %s into hash with coll length %s"
+             ;;          string (length collection))
+             ;; (fussy--print-hash-table fussy--all-cache)
+             (puthash string (cl-copy-list collection)
+                      fussy--all-cache))
+           (nconc collection (length prefix))))))))
 
 ;;
 ;; (@* "Scoring & Highlighting" )
@@ -671,7 +686,8 @@ This implementation uses `fzf-native-score-all' to do all its scoring in one go.
 
 Ignore CACHE. This is only added to match `fussy-score'."
   (when (fboundp 'fzf-native-score-all)
-    (fzf-native-score-all candidates string)))
+    (let ((string (fussy-encode-coding-string string)))
+      (fzf-native-score-all candidates string))))
 
 (defun fussy-score (candidates string &optional cache)
   "Score and propertize CANDIDATES using STRING.
@@ -680,19 +696,18 @@ Use CACHE for scoring.
 
 Set a text-property \='completion-score on candidates with their score.
 `completion--adjust-metadata' later uses this \='completion-score for sorting."
-  (let ((result '()))
+  (let ((result '())
+        (string (fussy-encode-coding-string
+                 (if (memq fussy-score-fn fussy-whitespace-ok-fns)
+                     string
+                   (replace-regexp-in-string "\\\s" "" string)))))
     (dolist (x candidates)
       (setf x (copy-sequence x))
       (if (> (length x) fussy-max-word-length-to-score)
           ;; Don't score x but don't filter it out either.
           (unless fussy-filter-unscored-candidates
             (push x result))
-        (let ((score (funcall fussy-score-fn
-                              x
-                              (if (memq fussy-score-fn fussy-whitespace-ok-fns)
-                                  string
-                                (replace-regexp-in-string "\\\s" "" string))
-                              cache)))
+        (let ((score (funcall fussy-score-fn x string cache)))
           ;; (message
           ;;  (format "fn: %S candidate: %s query: %s score %S"
           ;;          'fussy-score x string score))
@@ -708,7 +723,7 @@ Set a text-property \='completion-score on candidates with their score.
                x (funcall fussy-propertize-fn x score)))
             (push x result)))))
     ;; Returns nil if empty.
-    (reverse result)))
+    result))
 
 (defun fussy--should-propertize-p ()
   "Whether or not to call `fussy-propertize-fn'.
@@ -829,6 +844,16 @@ If SCORE does not have indices to highlight, return STR unmodified."
   "Set up `fussy'."
   (unless (memq 'fussy completion-styles)
     (push 'fussy completion-styles))
+
+  ;; https://github.com/minad/consult/issues/585
+  ;; https://github.com/axelf4/hotfuzz?tab=readme-ov-file#dynamic-module
+  (setq fussy-remove-bad-char-fn nil)
+  (with-eval-after-load 'consult
+    (defvar consult--tofu-char)
+    (defvar consult--tofu-range)
+    (setq consult--tofu-char #x100000
+          consult--tofu-range #x00fffe))
+
   ;; For example, project-find-file uses 'project-files which uses
   ;; substring completion by default. Set our own defaults.
   (setq completion-category-overrides
@@ -945,7 +970,7 @@ rebuild it here. We could also try caching the pattern instead of creating it
 again."
   (cond
    ((eq fussy-filter-fn 'fussy-filter-flex)
-    ;; This comes from `completion-substring--all-completions'
+    ;; This comes from `fussy-emacs-legacy-completion-substring--all-completions'
     ;; Look at `fussy-filter-flex'.
     (let* ((basic-pattern (completion-basic--pattern
                            beforepoint afterpoint bounds))
@@ -954,7 +979,7 @@ again."
                       (cons 'prefix basic-pattern)))
            (pattern
             (completion-pcm--optimize-pattern
-             (completion-flex--make-flex-pattern pattern))))
+             (fussy-emacs-legacy-completion-flex--make-flex-pattern pattern))))
       pattern))
    (:default ;; `fussy-filter-default'
     (fussy-make-pcm-highlight-pattern
@@ -1025,10 +1050,6 @@ See `fussy-remove-bad-char-fn'."
       (substring string 0 (- (length string) 1))
     string))
 
-(defsubst fussy-encode-coding-string (string)
-  "Call `encode-coding-string' for STRING."
-  (encode-coding-string string 'utf-8 t))
-
 (defun fussy--print-hash-table (table)
   "Print TABLE."
   (message "------------------------------------------------------------------")
@@ -1078,10 +1099,10 @@ Use `orderless' for filtering by passing STRING, TABLE and PRED to
 Respect PRED and POINT.  The filter here is the same as in
 `completion-flex-all-completions'."
   (pcase-let ((`(,completions ,pattern ,prefix ,_suffix ,_carbounds)
-               (completion-substring--all-completions
+               (fussy-emacs-legacy-completion-substring--all-completions
                 string
                 table pred point
-                #'completion-flex--make-flex-pattern)))
+                #'fussy-emacs-legacy-completion-flex--make-flex-pattern)))
     (list completions pattern prefix)))
 
 (defun fussy-filter-default (string table pred point)
@@ -1151,14 +1172,14 @@ Respect BEFOREPOINT, AFTERPOINT, and BOUNDS."
   (when (fussy--use-pcm-highlight-p)
     ;; Note to self:
     ;; The way we create the pattern here can be found in
-    ;; `completion-substring--all-completions'.
+    ;; `fussy-emacs-legacy-completion-substring--all-completions'.
     (let* ((basic-pattern (completion-basic--pattern
                            beforepoint afterpoint bounds))
            (pattern (if (not (stringp (car basic-pattern)))
                         basic-pattern
                       (cons 'prefix basic-pattern))))
       (completion-pcm--optimize-pattern
-       (completion-flex--make-flex-pattern pattern)))))
+       (fussy-emacs-legacy-completion-flex--make-flex-pattern pattern)))))
 
 ;;
 ;; (@* "Pattern Compiler" )
@@ -1308,7 +1329,7 @@ This is to try to avoid a additional sort step."
   "Score STR for QUERY with ARGS using `flx-rs-score'."
   (require 'flx-rs)
   (when (fboundp 'flx-rs-score)
-    (flx-rs-score (funcall fussy-remove-bad-char-fn str) query args)))
+    (flx-rs-score (fussy-without-bad-char str) query args)))
 
 (defun fussy-fuz-score (str query &rest _args)
   "Score STR for QUERY using `fuz'.
@@ -1318,10 +1339,7 @@ skim or clangd algorithm can be used.
 If `orderless' is used for filtering, we skip calculating matches
 for more speed."
   (require 'fuz)
-  (let ((str (funcall fussy-remove-bad-char-fn str))
-        (query
-         ;; Assume query can just be passed in as a unibyte string.
-         (fussy-encode-coding-string query)))
+  (let ((str (fussy-without-bad-char str)))
     (if fussy-fuz-use-skim-p
         (if (fussy--orderless-p)
             (when (fboundp 'fuz-calc-score-skim)
@@ -1349,10 +1367,7 @@ If `orderless' is used for filtering, we skip calculating matches
 for more speed."
   (require 'fuz-bin)
   ;; (message (format "before: str: %s query: %s" str query))
-  (let ((str (funcall fussy-remove-bad-char-fn str))
-        (query
-         ;; Assume query can just be passed in as a unibyte string.
-         (fussy-encode-coding-string query)))
+  (let ((str (fussy-without-bad-char str)))
     ;; (message (format "after: str: %s query: %s" str query))
     (if fussy-fuz-use-skim-p
         (if (fussy--orderless-p)
@@ -1376,7 +1391,7 @@ This should be paired with `fussy-filter-orderless' to obtain match
 highlighting."
   (require 'liquidmetal)
   (when (fboundp 'liquidmetal-score)
-    (list (liquidmetal-score str query))))
+    (list (liquidmetal-score (fussy-without-bad-char str) query))))
 
 ;; `sublime-fuzzy' integration
 (declare-function "sublime-fuzzy-score" "sublime-fuzzy")
@@ -1385,12 +1400,7 @@ highlighting."
   "Score STR for QUERY using `sublime-fuzzy'."
   (require 'sublime-fuzzy)
   (when (fboundp 'sublime-fuzzy-score)
-    (let ((str
-           (funcall fussy-remove-bad-char-fn str))
-          (query
-           ;; Assume query can just be passed in as a unibyte string.
-           (fussy-encode-coding-string query)))
-      (list (sublime-fuzzy-score query str)))))
+    (list (sublime-fuzzy-score query (fussy-without-bad-char str)))))
 
 ;; `fzf-native' integration
 (defvar fussy--fzf-native-slab nil)
@@ -1404,9 +1414,8 @@ highlighting."
   "Score STR for QUERY using `fzf-native'."
   (require 'fzf-native)
   (when (fboundp 'fzf-native-score)
-    (let ((str (funcall fussy-remove-bad-char-fn str))
-          (query (fussy-encode-coding-string query)))
-      (fzf-native-score str query (fussy--fzf-native-slab)))))
+    (fzf-native-score
+     (fussy-without-bad-char str) query (fussy--fzf-native-slab))))
 
 ;; `hotfuzz' integration
 (declare-function "hotfuzz--cost" "hotfuzz")
@@ -1419,5 +1428,50 @@ highlighting."
     ;; See `hotfuzz-all-completions'.
     (list (+ 10000 (- (hotfuzz--cost query str))))))
 
+;;; Legacy
+
+;; This package uses internal Emacs functions.  Some of these functions
+;; were modified in https://github.com/emacsmirror/emacs/commit/aa181cd35220,
+
+(defun fussy-emacs-legacy-completion-flex--make-flex-pattern (pattern)
+  "Convert PCM-style PATTERN into PCM-style flex pattern.
+
+This turns
+    (prefix \"foo\" point)
+into
+    (prefix \"f\" any \"o\" any \"o\" any point)
+which is at the core of flex logic.  The extra
+`any' is optimized away later on."
+  (mapcan (lambda (elem)
+            (if (stringp elem)
+                (mapcan (lambda (char)
+                          (list (string char) 'any))
+                        elem)
+              (list elem)))
+          pattern))
+
+(defun fussy-emacs-legacy-completion-substring--all-completions
+    (string table pred point &optional transform-pattern-fn)
+  "Match the presumed substring STRING to the entries in TABLE.
+Respect PRED and POINT.  The pattern used is a PCM-style
+substring pattern, but it be massaged by TRANSFORM-PATTERN-FN, if
+that is non-nil."
+  (let* ((beforepoint (substring string 0 point))
+         (afterpoint (substring string point))
+         (bounds (completion-boundaries beforepoint table pred afterpoint))
+         (suffix (substring afterpoint (cdr bounds)))
+         (prefix (substring beforepoint 0 (car bounds)))
+         (basic-pattern (completion-basic--pattern
+                         beforepoint afterpoint bounds))
+         (pattern (if (not (stringp (car basic-pattern)))
+                      basic-pattern
+                    (cons 'prefix basic-pattern)))
+         (pattern (completion-pcm--optimize-pattern
+                   (if transform-pattern-fn
+                       (funcall transform-pattern-fn pattern)
+                     pattern)))
+         (all (completion-pcm--all-completions prefix pattern table pred)))
+    (list all pattern prefix suffix (car bounds))))
+
 (provide 'fussy)
 ;;; fussy.el ends here
